const int claw_num = 46;
const int slide_num = 44;

const int pingPin = 5; // Trigger Pin of Ultrasonic Sensor
const int echoPin = 6; // Echo Pin of Ultrasonic Sensor

int closed_pos = 75;
int open_pos = 0;
int forward_pos = 100;
int backward_pos = 180;

#include <Servo.h>

Servo claw;
Servo slide;

char receivedChar;
boolean newData = false;

/*
Functions to control the intialization of the DC Motors, Servos, and Sensors.
*/

void intializeMotors() {
  pinMode(8, OUTPUT);
  pinMode(9, OUTPUT);
  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);
  
  digitalWrite(8, HIGH);
  digitalWrite(11, LOW);
  digitalWrite(12, HIGH);
  digitalWrite(13, LOW);
  
  analogWrite(9, 0);
  analogWrite(10, 0);
}

void initializeServos() {
  claw.attach(claw_num);
  slide.attach(slide_num);
  claw.write(open_pos);
  slide.write(backward_pos);
}

void initializeSensors() {
  pinMode(31, OUTPUT);
  pinMode(33, OUTPUT);
  
  pinMode(A9, INPUT);
  pinMode(A11, INPUT);

  pinMode(pingPin, OUTPUT);
  pinMode(echoPin, INPUT);
}

void setup() {
  Serial.begin(115200);
  intializeMotors();
  initializeServos();
  initializeSensors();

  pinMode(53, OUTPUT);
}

/*
Functions to control the claw.
*/

void clawClose(int time_) {
  claw.write(closed_pos);
  delay(time_);
}

void clawOpen(int time_) {
  claw.write(open_pos);
  delay(time_);
}

void goForward(int time_) {
  slide.write(forward_pos);
  delay(time_);  
}

void goBackward(int time_) {
  slide.write(backward_pos);
  delay(time_);  
}

void incrementForward(int time_) {
  int current_pos = slide.read();
  slide.write(current_pos - 5);
  delay(time_);  
}

void incrementBackward(int time_) {
  int current_pos = slide.read();
  slide.write(current_pos + 5);
  delay(time_);  
}

/*
Functions to control the turntable rotation.
*/

void rotateClockwise(int speed_) {
  digitalWrite(12, HIGH);
  digitalWrite(13, LOW);
  analogWrite(10, speed_);
}

void rotateCounterclockwise(int speed_) {
  digitalWrite(12, LOW);
  digitalWrite(13, HIGH);
  analogWrite(10, speed_);
}

void stopRotation() {
  analogWrite(10, 0);
}

/*
Functions to control the leadscrew movement.
*/

void goUp(int speed_) {
  if (!touch_2_IsPressed()) {
    digitalWrite(8, HIGH);
    digitalWrite(11, LOW);
    analogWrite(9, speed_);  
  } else {
    noGo();  
  }
}

void goDown(int speed_) {
  if (!touch_1_IsPressed()) {
    digitalWrite(8, LOW);
    digitalWrite(11, HIGH);
    analogWrite(9, speed_);
  } else {
    noGo();
  }
}

void noGo() {
  analogWrite(9, 0);
}

/*
Function to process Serial information from the Raspberry Pi.
*/

void recvInfo() {
  if (Serial.available() > 0) {
    receivedChar = Serial.read();
    newData = true;    
  }  
}

/*
Functions to handle movement according to the input from the Raspberry Pi.
*/

void handleRotation() {
  if (receivedChar == '1') {
    rotateClockwise(100); 
  } else if (receivedChar == '2') {
    rotateCounterclockwise(100);
  } else if (receivedChar == '3') {
    stopRotation();
  }
  delay(50);
}

void handleLeadscrew() {
  if (receivedChar == '4') {
    goUp(255); 
  } else if (receivedChar == '5') {
    goDown(255);
  } else if (receivedChar == 'B') {
    goUp(125);
  } else if (receivedChar == 'C') {
    goDown(125);
  } else if (receivedChar == '6') {
    noGo();
  }
  delay(50);   
}

void handleClaw() {
  if (receivedChar == '0') {
    if (claw.read() >= 75) {
      clawOpen(500);
    } else {
      clawClose(500);
    } 
  }
  delay(50);
}

void handleSlide() {
  if (receivedChar == '7') {
    goForward(1000); 
  } else if (receivedChar == '8') {
    goBackward(1000);
  }
  delay(50);
}

void handleIncrements() {
  if (receivedChar == '9') {
    incrementForward(25);
  } else if (receivedChar == 'A') {
    incrementBackward(25);
  }
  delay(25);
}

void goUpSpecifiedAmount() {
  if (receivedChar == 'B') {
    goUp(100);
    delay(200);
    noGo();
  } 
}

/*
Functions to control the data from the Ultrasonic Sensor.
*/

int returnDistance() {
  long cm = 0;
  while (cm == 0) {
    long duration;
    digitalWrite(pingPin, LOW);
    delayMicroseconds(2);
    digitalWrite(pingPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(pingPin, LOW);
    duration = pulseIn(echoPin, HIGH);
    cm = microsecondsToCentimeters(duration);
    delay(10);  
  }
  return cm; 
}

long microsecondsToCentimeters(long microseconds) {
   return microseconds / 29 / 2;
}

void changeInDistance(int distance1, int distance2) {
  if (abs(distance1 - distance2) > 2) {
    digitalWrite(31, HIGH);
    digitalWrite(33, HIGH);
  }
}

/*
Functions to control the data from the Touch Sensor.
*/

boolean touch_1_IsPressed() {
  if (analogRead(A9) < 550) {
    digitalWrite(31, HIGH);
    return true;    
  } else {
    digitalWrite(31, LOW);
    return false;
  }
}

boolean touch_2_IsPressed() {
  if (analogRead(A11) < 500) {
    digitalWrite(33, HIGH);
    return true;    
  } else {
    digitalWrite(33, LOW);
    return false;
  }
}

void loop() {
  
  //int distance1 = returnDistance();
  
  //Serial.println(returnDistance());
//  
  //Serial.println(analogRead(A9)); //>600 is not touched, < 450 is pressed
  //Serial.println(analogRead(A11)); //>600 is not touched, < 450 is pressed
  
//  if (analogRead(A9) < 600) {
//    digitalWrite(31, HIGH);
//  } else if (analogRead(A11) < 600) {
//    digitalWrite(33, HIGH);
//  } else {
//    digitalWrite(31, LOW);
//    digitalWrite(33, LOW);
//  }

  recvInfo(); 
  while(newData == true) {
    Serial.print(receivedChar);
    newData = false;
    digitalWrite(53, HIGH);
    handleRotation();
    handleLeadscrew();
    handleClaw();
    handleSlide();
    handleIncrements();
    goUpSpecifiedAmount();
  }

  digitalWrite(53, LOW);

  //touch_1_IsPressed();
  //touch_2_IsPressed();

//  int distance2 = returnDistance();
//  changeInDistance(distance1, distance2);
//  delay(50);
  digitalWrite(31, LOW);
  digitalWrite(33, LOW);
}
